// $Id: UFL_lp.hpp,v 1.2 2003/12/03 01:46:35 magh Exp $

/* ------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

#ifndef _UFL_LP_H
#define _UFL_LP_H

#include "BCP_lp_user.hpp"
#include "BCP_parameters.hpp"
#include "UFL_lp_param.hpp"

/*---------------------------------------------------------------------*/
class UFL_lp : public BCP_lp_user {
public:
  UFL_lp() : lp_par(), M(0), N(0), demand(0), infinity_(0.0) {};
  ~UFL_lp() {};
  BCP_parameter_set<UFL_lp_par> lp_par;  

private:
  virtual OsiSolverInterface * initialize_solver_interface();
  virtual void unpack_module_data(BCP_buffer & buf);
  virtual void pack_cut_algo(const BCP_cut_algo * cut, BCP_buffer & buf);
  virtual BCP_cut_algo * unpack_cut_algo(BCP_buffer & buf);
  virtual void generate_cuts_in_lp(const BCP_lp_result& lpres,
				   const BCP_vec<BCP_var*>& vars,
				   const BCP_vec<BCP_cut*>& cuts,
				   BCP_vec<BCP_cut*>& new_cuts,
				   BCP_vec<BCP_row*>& new_rows);
  virtual void display_lp_solution(const BCP_lp_result& lpres,
				   const BCP_vec<BCP_var*>& vars,
				   const BCP_vec<BCP_cut*>& cuts,    
				   const bool final_lp_solution);
  virtual void modify_lp_parameters(OsiSolverInterface* lp, 
				    bool in_strong_branching);
  virtual void cuts_to_rows(const BCP_vec<BCP_var*>& vars,
			    BCP_vec<BCP_cut*>& cuts,       
			    BCP_vec<BCP_row*>& rows,       
			    const BCP_lp_result& lpres,
			    BCP_object_origin origin, bool allow_multiple);
private:
  int M;               //number of clients (index on i)
  int N;               //number of depots  (index in j)
  double * demand;     //d[j]
  double infinity_;
  
  inline int xindex(int i, int j){return i*N + j;}
  inline int yindex(int j){return (M * N) + j;}
};

#endif
