// $Id: UFL_tm.hpp,v 1.2 2003/12/03 01:46:35 magh Exp $

/* ------------------------------------------------------------------------
 Author: Matthew Galati (magh@lehigh.edu)

 (c) Copyright 2003 Lehigh University. All Rights Reserved.

 This software is licensed under the Common Public License. Please see 
 accompanying file for terms.    
---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/
#ifndef _UFL_TM_H
#define _UFL_TM_H

#include "BCP_parameters.hpp"
#include "BCP_tm_user.hpp"
#include "UFL_tm_param.hpp"
#include "UFL_lp_param.hpp"

/*---------------------------------------------------------------------*/
class UFL_tm : public BCP_tm_user {
public:
  UFL_tm() : tm_par(), lp_par(), M(0), N(0), trans_cost(0),
	     fixed_cost(0), demand(0), total_demand(0.0) {};
  ~UFL_tm();
  BCP_parameter_set<UFL_tm_par> tm_par;//TM parameter list
  BCP_parameter_set<UFL_lp_par> lp_par;//LP parameter list
  
private:  
  virtual void pack_module_data(BCP_buffer& buf, BCP_process_t ptype);
  virtual void pack_cut_algo(const BCP_cut_algo* cut, BCP_buffer& buf);
  virtual BCP_cut_algo* unpack_cut_algo(BCP_buffer& buf);
  virtual void initialize_core(BCP_vec<BCP_var_core*>& vars,
			       BCP_vec<BCP_cut_core*>& cuts,
			       BCP_lp_relax*& lp_relax);
private:	
  int M;               //number of clients (index on i)
  int N;               //number of depots  (index in j)
  double * trans_cost; //c[i][j] -> c[xindex(i,j)]
  double * fixed_cost; //f[j]
  double * demand;     //d[j]
  double total_demand; //sum{j in N} d[j]

  inline int xindex(const int i, const int j) {return i*N + j;}
  inline int yindex(const int j)              {return M*N + j;}
  void read_data(const char* datafile);
};

#endif
